#!/usr/bin/ruby
require 'pathname'
require 'open-uri'
require 'zip'
require 'xcodeproj'

  module DTRipperProjectCreator
    #main project part
    TEMPLETE_MAIN_PROJECT_ZIP_FILE_NAME = "DTRipperMainTemplete-master.zip"
    TEMPLETE_MAIN_PROJECT_DOWNLOAD_URL  = "https://github.com/NicolasKim/DTRipperMainTemplete/archive/master.zip"
    TEMPLETE_MAIN_PROJECT_ROOT_FOLDER   = "DTRipperMainTemplete-master"
    MAIN_PROJECT_REPLACEABLE_STRING_REX = /(\#{2}|_{2}|-{2})TempleteMainProject(\#{2}|_{2}|-{2})/

    #module project part
    TEMPLETE_MODULE_PROJECT_ZIP_FILE_NAME = "DTRipperModuleTemplete-master.zip"
    TEMPLETE_MODULE_PROJECT_DOWNLOAD_URL  = "https://github.com/NicolasKim/DTRipperModuleTemplete/archive/master.zip"
    TEMPLETE_MODULE_PROJECT_ROOT_FOLDER   = "DTRipperModuleTemplete-master"
    REPLACEABLE_STRING_REX                = /(\#{2}|_{2}|-{2})TempleteModule(\#{2}|_{2}|-{2})/

    class ProjectReplacer
      def initialize(proj_name,proj_dir,zip_file_name,down_load_url,proj_root_folder,replace_rex)
        @proj_name = proj_name
        @proj_dir  = proj_dir
        @zip_file_name = zip_file_name
        @down_load_url = down_load_url
        @proj_root_folder = proj_root_folder
        @replace_rex      = replace_rex
      end
      def replace
        if File.exists?(File.join(@proj_dir,@proj_name))
          puts "Already have project named '#{@proj_name}',If you want to create new one,delete old one please"
          return false
        end


        unzip_file_path = File.join(@proj_dir,@zip_file_name)
        if File.exists?(unzip_file_path)
          puts "templete project zip file is already exist"
          File.delete(unzip_file_path)
        end

        puts "Downloading templete project...."
        File.open(unzip_file_path, 'wb') {|f| f.write(open(@down_load_url) {|f1| f1.read})}
        puts "Templete project download success!!"

        puts "Start unzip templete project...."

        unzip(unzip_file_path)

        puts "Unzip templete project success!!"

        File.delete(unzip_file_path)

        puts "Delete templete project zip file success!!"


        File.rename(File.join(@proj_dir,@proj_root_folder),File.join(@proj_dir,@proj_name))

        replace_all_replacable_str File.join(@proj_dir,@proj_name)

        puts "Replace finished!!"
        return true
      end
      def replace_all_replacable_str(path)
        dir = Dir.new(path)
        dir.each do |sub_dir|

          if sub_dir == "." || sub_dir == ".."
            next
          end


          if sub_dir.to_s.match(@replace_rex)
            changed_path = File.join(path,sub_dir.to_s.gsub(@replace_rex,@proj_name))
            File.rename(File.join(path,sub_dir.to_s),changed_path)
            if File.ftype(changed_path) == "file"
              file_content_replace changed_path
            else
              replace_all_replacable_str changed_path
            end
          else
            orgin_path = File.join(path,sub_dir)
            if File.ftype(orgin_path) == "file"
              file_content_replace orgin_path
            elsif
            replace_all_replacable_str orgin_path
            end

          end




        end
      end
      def file_content_replace(path)
        File.open(path,"r:utf-8") do |fr|
          buffer = fr.read.gsub(@replace_rex,@proj_name)
          # puts buffer
          File.open(path, "w:utf-8") { |fw| fw.write(buffer) }
        end
      end
      def unzip(path)
        full_path = File.expand_path(path)
        entries = get_entries_name(path)
        Zip::File.open(full_path) do |f|
          entries.each do |entry|
            f.extract(entry["entry"],entry["dest_path"]) {true}
            #puts "unzip #{entry} succeed!"
          end
        end
        rescue =>e
        puts(e)
        exit 3
      end
      def get_entries_name(path)
        full_path = File.expand_path(path)
        entries = []
        Zip::InputStream::open(full_path) do |io|
          while (entry = io.get_next_entry)
            h = Hash["entry"=>entry.name , "dest_path"=>File.join(@proj_dir,entry.name)]
            entries << h
          end
        end
        entries
      end
    end

    class DTMainProjectGenerator
      def initialize(proj_name,proj_dir=File.dirname(__FILE__))
        @proj_name = proj_name
        @proj_dir  = proj_dir
        @replacer = ProjectReplacer.new(proj_name,proj_dir,
                                        TEMPLETE_MAIN_PROJECT_ZIP_FILE_NAME,
                                        TEMPLETE_MAIN_PROJECT_DOWNLOAD_URL,
                                        TEMPLETE_MAIN_PROJECT_ROOT_FOLDER,
                                        MAIN_PROJECT_REPLACEABLE_STRING_REX)
      end

      def create
        return @replacer.replace
      end

      def self.create_workspace(proj_dir,proj_name)
        proj_path = File.join(proj_dir,proj_name,"#{proj_name}.xcodeproj")
        proj_file_ref = Xcodeproj::Workspace::FileReference.new("#{proj_name}.xcodeproj")
        workspace = nil
        workspace_path = File.join(proj_dir,proj_name,"#{proj_name}.xcworkspace")
        if File.exists? workspace_path
          workspace = Xcodeproj::Workspace.new_from_xcworkspace(workspace_path)
        else
          workspace = Xcodeproj::Workspace.new(nil, proj_file_ref)
        end
        workspace
      end



    end

    class DTModuleGenerator
      def initialize(module_name,module_dir=File.dirname(__FILE__),main_proj_name="MainProject", main_proj_dir=nil)
        @module_dir  = module_dir
        @module_name = module_name
        @main_proj_dir = main_proj_dir
        @main_proj_name = main_proj_name
        if main_proj_dir == nil
          @main_proj_dir = module_dir
        end
      end

      def config
        workspace_file_dir = File.join(@main_proj_dir,@main_proj_name,"#{@main_proj_name}.xcworkspace")
        main_proj_file_dir = File.join @main_proj_dir ,@main_proj_name , "#{@main_proj_name}.xcodeproj"
        module_proj_file_dir = File.join(@module_dir,@module_name,"#{@module_name}.xcodeproj")
        podfile_dir          = File.join @main_proj_dir ,@main_proj_name , "Podfile"

        if !File.exists?(main_proj_file_dir)
          puts "Did not found main project at path:'#{main_proj_file_dir}'"
          return
        end


        File.open(podfile_dir,"r:utf-8") do |fr|
          buffer = fr.read
          if !buffer.include?("#{@module_name}")
            endIndex = buffer.index("end")
            puts endIndex
            buffer.insert(endIndex,"  pod '#{@module_name}', :path => '#{File.join(@module_dir,@module_name)}' \n")
            File.open(podfile_dir, "w:utf-8") { |fw|
              fw.write(buffer)
            }
          end
        end
        workspace = DTMainProjectGenerator.create_workspace(@main_proj_dir,@main_proj_name)
        workspace << module_proj_file_dir
        workspace.save_as workspace_file_dir
        exec "pod install --project-directory=#{File.join(@main_proj_dir,@main_proj_name)}"
      end

      def create

        main_proj_file_dir = File.join @main_proj_dir ,@main_proj_name , "#{@main_proj_name}.xcodeproj"
        if !File.exists? main_proj_file_dir
          #create main project
          return DTMainProjectGenerator.new(@main_proj_name,@main_proj_dir).create
        end

        #create module project
        result = ProjectReplacer.new(@module_name,@module_dir,
                                      TEMPLETE_MODULE_PROJECT_ZIP_FILE_NAME,
                                      TEMPLETE_MODULE_PROJECT_DOWNLOAD_URL,
                                      TEMPLETE_MODULE_PROJECT_ROOT_FOLDER,
                                      REPLACEABLE_STRING_REX).replace
        if result
          config
        end
      end



    end

  end



